# import packages/modules
import pyThermoLinkDB as ptdblink
import pyThermoDB as ptdb
import os
from pprint import pprint as pp

# local

# check version
print(ptdblink.__version__)
print(ptdb.__version__)
# author
print(ptdblink.__author__)

# =======================================
# ! LOAD THERMODB
# =======================================

# ! ethanol
# thermodb file name
EtOH_thermodb_file = os.path.join(os.getcwd(), 'test', 'ethanol.pkl')
# load
EtOH_thermodb = ptdb.load_thermodb(EtOH_thermodb_file)
print(type(EtOH_thermodb))

EtOH_thermodb

# check
pp(EtOH_thermodb.check())
# check properties
pp(EtOH_thermodb.check_properties())
# datastrcuture
pp(EtOH_thermodb.check_property('GENERAL').data_structure())

# ! methanol
# thermodb file name
MeOH_thermodb_file = os.path.join(os.getcwd(), 'test', 'methanol.pkl')
# load
MeOH_thermodb = ptdb.load_thermodb(MeOH_thermodb_file)
print(type(MeOH_thermodb))

MeOH_thermodb

# check
pp(MeOH_thermodb.check())

# ! CO2
CO2_thermodb_file = os.path.join(
    os.getcwd(), 'test', 'Carbon Dioxide-multiple.pkl')
# load
CO2_thermodb = ptdb.load_thermodb(CO2_thermodb_file)
print(type(CO2_thermodb))

# check
pp(CO2_thermodb.check())
pp(CO2_thermodb.check_property('GENERAL').data_structure())
pp(CO2_thermodb.check_property('GENERAL-2').data_structure())

# =======================================
# ! THERMODB LINK CONFIGURATION
# =======================================
# init thermodb hub
thub1 = ptdblink.thermodb_hub()
print(type(thub1))

# add component thermodb
thub1.add_thermodb('EtOH', EtOH_thermodb)
thub1.add_thermodb('MeOH', MeOH_thermodb)
thub1.add_thermodb('CO2', CO2_thermodb)

# * add thermodb rule
thermodb_config_file = os.path.join(os.getcwd(), 'test', 'thermodb_config.yml')
# one component
# thub1.config_thermodb_rule(thermodb_config_file, name='EtOH')
# all components
thub1.config_thermodb_rule(thermodb_config_file, name='ALL')

# thermodb
# pp(thub1.thermodb)

# thermodb rules
# pp(thub1.thermodb_rule)

# get components
pp(thub1.get_components())

# build
datasource, equationsource = thub1.build()
pp(datasource)
pp(equationsource)

# hub
pp(thub1.hub)

# check
# pp(thub1.check())

# =======================================
# ! TEST
# =======================================
# data
pp(datasource['CO2']['Pc']['value'])

# equation
pp(equationsource['CO2']['VaPr'].args)
pp(equationsource['CO2']['VaPr'].cal(T=298.15))



# THERMODB

# import packages/modules
import pyThermoDB
import os
import yaml
# local
from .thermolink import ThermoLink


class ThermoDBHub(ThermoLink):
    # vars
    _thermodb = {}
    _thermodb_rule = {}
    _hub = {}

    def __init__(self):
        # init super class
        ThermoLink().__init__()

    @property
    def thermodb(self):
        return self._thermodb

    @property
    def thermodb_rule(self):
        return self._thermodb_rule

    @property
    def hub(self):
        return self._hub

    def get_components(self):
        '''
        Gets a component list

        Parameters
        ----------
        None

        Returns
        -------
        components : list
            list of components
        '''
        try:
            components = list(self._thermodb.keys())
            return components
        except Exception as e:
            raise Exception('Getting components failed!, ', e)

    def config_thermodb_rule(self, config_file, name="ALL", disp=False) -> bool:
        '''
        Configs thermodb rule defined for each component

        Parameters
        ----------
        config_file: str
            config file path
        name: str
            name of the record, optional, default value is ALL.
        disp: bool
            display the config log

        Returns
        -------
        True : bool
            success

        Notes
        -----
        config_file is a yml file format as:
        ```
        EtOH:
            Pc: ['GENERAL','Pc']
            Tc: ['GENERAL','Tc']
            AcFa: ['GENERAL','AcFa']
            VaPr: ['VAPOR-PRESSURE','VaPr']
        MeOH:
            Pc: ['GENERAL','Pc']
            Tc: ['GENERAL','Tc']
            AcFa: ['GENERAL','AcFa']
            VaPr: ['VAPOR-PRESSURE','VaPr']
        ```
        '''
        try:
            # check
            if config_file:
                # check file
                if not os.path.exists(config_file):
                    raise Exception('Configuration file not found!')

                # set name
                name = str(name).strip()
                # load
                with open(config_file, 'r') as f:
                    _ref = yaml.load(f, Loader=yaml.FullLoader)

                    # check
                    if name == "ALL":
                        # check name exists
                        if _ref.keys():
                            # looping through
                            for key in _ref.keys():
                                # get record
                                record_thermodb_rule = _ref[key]

                                # check key exists
                                if key in self._thermodb.keys():
                                    # add
                                    self._thermodb_rule[key].update(
                                        record_thermodb_rule)

                                    # check disp
                                    if disp:
                                        print(
                                            f'{key} thermodb rule successfully registered.')
                                else:
                                    print(
                                        f'{key} not found, no thermodb provided!')
                            # res
                            return True
                        else:
                            raise Exception('Record not found!')
                    else:
                        # check name exists
                        if name in _ref.keys():
                            # get record
                            record_thermodb_rule = _ref[name]

                            # check name exists
                            if name in self._thermodb.keys():
                                # looping through
                                self._thermodb_rule[name].update(
                                    record_thermodb_rule)

                                # check disp
                                if disp:
                                    print(
                                        f'{name} thermodb rule successfully registered.')
                                # res
                                return True
                            else:
                                print(
                                    f'{name} not found, no thermodb provided!')
                                # res
                                return False
                        else:
                            print(f'{name} not found, no thermodb provided!')
                            return False

        except Exception as e:
            raise Exception('Configuration failed!, ', e)

    def add_thermodb_rule(self, rules):
        '''
        Adds a thermodb rule dict

        Parameters
        ----------
        rules: dict
            thermodb rule dict for all components

        Returns
        -------
        True : bool
            success

        Notes
        -----
        - A dictionary file with the following format:

        {
            'CO2':
                'Pc': ['GENERAL','Pc']
                'Tc': ['GENERAL','Tc']
                'AcFa': ['GENERAL','AcFa']
                'VaPr': ['VAPOR-PRESSURE','VaPr'],
        }
        '''
        try:
            for key, value in rules.items():
                # check key exist
                if key not in self._thermodb.keys():
                    # log warning
                    print(f"{key} is not in thermodb!")
                    continue

                # add
                self._thermodb_rule[key] = value
                # res
                return True
            # res
            return True
        except Exception as e:
            raise Exception('Adding new rule failed!, ', e)

    def delete_thermodb_rule(self, name):
        '''
        Deletes the current thermodb rule

        Parameters
        ----------
        name: str
            name of the record

        Returns
        -------
        True : bool
            success
        '''
        try:
            # check key exist
            if name not in self._thermodb_rule.keys():
                # log warning
                print(f"{name} is not in thermodb_rule!")
                return False

            # del
            del self._thermodb_rule[name]
            # res
            return True
        except Exception as e:
            raise Exception('Deleting rule failed!, ', e)

    def add_thermodb(self, name, data: pyThermoDB.docs.compbuilder.CompBuilder) -> bool:
        '''
        Adds new thermodb such as: CO2_thermodb

        Parameters
        ----------
        name: str
            name of the record
        data: pyThermoDB.docs.compbuilder.CompBuilder
            data of the record

        Returns
        -------
        True : bool
            success
        '''
        try:
            self._thermodb[name] = data
            # create thermodb rule
            self._thermodb_rule[name] = {}
            # res
            return True
        except Exception as e:
            raise Exception('Adding new record failed!, ', e)

    def update_thermodb(self, name, data: pyThermoDB.docs.compbuilder.CompBuilder) -> bool:
        '''
        Updates existing record

        Parameters
        ----------
        name: str
            name of the record
        data: pyThermoDB.docs.compbuilder.CompBuilder
            data of the record

        Returns
        -------
        True : bool
            success
        '''
        try:
            self._thermodb[name] = data
            # res
            return True
        except Exception as e:
            raise Exception('Updating record failed!, ', e)

    def delete_thermodb(self, name: str) -> bool:
        '''
        Deletes existing record

        Parameters
        ----------
        name: str
            name of the record

        Returns
        -------
        True : bool
            success
        '''
        try:
            del self._thermodb[name]
            # delete thermodb rule
            del self._thermodb_rule[name]
            # res
            return True
        except Exception as e:
            raise Exception('Deleting record failed!, ', e)

    def info_thermodb(self, name) -> dict:
        '''
        Gets info of thermodb

        Parameters
        ----------
        name: str
            name of the record

        Returns
        -------
        res : CompBuilder
            thermodb check
        '''
        try:
            # check key exist
            if name not in self._thermodb.keys():
                # log warning
                print(f"{name} is not in thermodb!")
                return None

            # res
            res = self._thermodb[name].check()
            return res
        except Exception as e:
            raise Exception('Getting info of record failed!, ', e)

    def build(self):
        '''
        Builds a datasource

        Parameters
        ----------
        None

        Returns
        -------
        datasource : dict
            datasource
        equationsource : dict
            equationsource

        Notes
        -----
        - A dictionary contains data and equations
        '''
        try:
            # components
            components = list(self._thermodb.keys())
            # datasource
            datasource = self._set_datasource(
                self._thermodb, self._thermodb_rule, components)
            # equationsource
            equationsource = self._set_equationsource(
                self._thermodb, self._thermodb_rule, components)

            # reset
            self._hub = {}
            # update hub

            # for each component
            for component in components:
                _data = datasource[component]
                _eq = equationsource[component]
                # save
                self._hub[component] = {**_data, **_eq}
            # res
            return datasource, equationsource
        except Exception as e:
            raise Exception('Building data/equation source failed!, ', e)

    def check(self):
        '''
        Checks data/equation source

        Parameters
        ----------
        None

        Returns
        -------
        thermodb : dict
            thermodb
        '''
        try:
            return self._hub
        except Exception as e:
            raise Exception('Checking data/equation source failed!, ', e)



# import packages/modules
import os
# local
from .config import __version__, __author__, __description__
from .docs import ThermoDBHub, ThermoLink


def get_version() -> str:
    '''
    Get the current version
    '''
    return __version__


def get_author() -> str:
    '''
    Get the author
    '''
    return __author__


def des() -> str:
    '''
    Get the description
    '''
    return __description__


def thermodb_hub() -> ThermoDBHub:
    '''
    Init thermolink

    Parameters
    ----------
    None

    Returns
    -------
    ThermoDBHub : ThermoDBHub
        A thermolink object
    '''
    try:
        # init thermolink
        ThermoDBHubC = ThermoDBHub()
        return ThermoDBHubC
    except Exception as e:
        raise Exception("Error: {}".format(e))


if __name__ == "__main__":
    print(des())



# THERMO LINK

# import packages/modules

# local


class ThermoLink:

    def __init__(self):
        # load reference
        pass

    def _set_datasource(self, thermodb: dict, thermodb_rule: dict, components: list) -> dict:
        '''
        Sets a datasource

        Parameters
        ----------
        thermodb : dict
            thermodb
        thermodb_rule : dict
            thermodb rule
        components : list
            list of components

        Returns
        -------
        datasource : dict
            datasource
        '''
        try:
            # datasource
            datasource = {}
            for component in components:
                if component in thermodb:
                    # set
                    datasource[component] = {}

                    # component registered data/equations
                    data = list(
                        thermodb[component].check_properties().keys())

                    # check
                    if len(data) != 0:
                        # looping through each data source (GENERAL)
                        for src in data:
                            # take data
                            df_src = thermodb[component].check_property(
                                src).data_structure()
                            # take all symbols
                            symbols = df_src['SYMBOL'].tolist()
                            # looping through item data
                            for symbol in symbols:
                                # check
                                if symbol is not None and symbol != 'None':
                                    # symbol
                                    symbol = str(symbol).strip()
                                    # val
                                    _val = thermodb[component].check_property(
                                        src).get_property(str(symbol).strip())

                                    # check symbol rename is required
                                    if symbol in thermodb_rule[component]['DATA']:
                                        # rename
                                        symbol = thermodb_rule[component]['DATA'][symbol]

                                    # update
                                    datasource[component][symbol] = _val
            # res
            return datasource
        except Exception as e:
            raise Exception('Building datasource failed!, ', e)

    def _set_equationsource(self, thermodb: dict, thermodb_rule: dict, components: list) -> dict:
        '''
        Sets equation source

        Parameters
        ----------
        thermodb : dict
            thermodb
        thermodb_rule : dict
            thermodb rule
        components : list
            list of components

        Returns
        -------
        datasource : dict
            datasource
        '''
        try:
            # datasource
            datasource = {}
            for component in components:
                if component in thermodb:
                    # set
                    datasource[component] = {}

                    # component registered data/equations
                    eq_data = list(
                        thermodb[component].check_functions().keys())

                    # check
                    if len(eq_data) != 0:
                        # parms
                        for eq in eq_data:
                            # get function structure
                            # eq_str = thermodb[component].get_function(
                            #     eq).eq_structure(1)

                            # symbol
                            symbol = str(eq).strip()
                            # val
                            _val = thermodb[component].check_function(
                                eq)

                            # check symbol rename is required
                            if symbol in thermodb_rule[component]['EQUATIONS']:
                                # rename
                                symbol = thermodb_rule[component]['EQUATIONS'][symbol]

                            datasource[component][symbol] = _val
            # res
            return datasource
        except Exception as e:
            raise Exception('Building datasource failed!, ', e)



from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

APP_NAME = 'PyThermoLinkDB'
VERSION = '1.0.0'
DESCRIPTION = 'PyThermoLinkDB is a Python package providing a robust and efficient interface between `PyThermoDB` and other applications.'
LONG_DESCRIPTION = "PyThermoLinkDB is a Python package providing a robust and efficient interface between `PyThermoDB` and other applications. It enables seamless thermodynamic data exchange, integration, and analysis. With PyThermoLinkDB, developers can easily link PyThermoDB to various tools, frameworks, and databases, streamlining thermodynamic workflows."

# Setting up
setup(
    name=APP_NAME,
    version=VERSION,
    author="Sina Gilassi",
    author_email="<sina.gilassi@gmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(exclude=['tests', '*.tests', '*.tests.*']),
    include_package_data=True,  # Make sure to include non-Python files
    # Add both config and data files
    package_data={'': ['config/*.yml', 'data/*.csv']},
    license='MIT',
    install_requires=['pandas', 'pillow', 'requests',
                      'urllib3', 'matplotlib', 'numpy', 'PyYAML', 'PyThermoDB'],
    keywords=['python', 'chemical engineering', 'thermodynamics',
              'PyThermoDB'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Education",
        "Programming Language :: Python :: 3.9",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ],
    python_requires='>=3.6',
)
